import 'package:flutter/material.dart';
import 'package:camera/camera.dart';
import 'package:signage_app/const/config.dart';
import 'package:signage_app/const/colors.dart';
import 'package:signage_app/const/dimens.dart';
import 'package:signage_app/widget/overlay_text.dart';

class Camera extends StatefulWidget {
  const Camera(
      {super.key, this.size = const Size(double.infinity, double.infinity)});

  final Size size;

  @override
  CameraState createState() {
    return CameraState();
  }
}

class CameraState extends State<Camera> {
  Future<void>? _initializeCameraController;
  CameraController? _controller;
  bool _availableCamera = true;

  @override
  void initState() {
    super.initState();
    initCamera();
  }

  @override
  void dispose() {
    super.dispose();
    _controller?.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final Size widgetSize = MediaQuery.of(context).size;
    final Size overlayTextContainerSize = Size(
        widgetSize.width * overlayTextWidthRatio,
        widgetSize.height * overlayTextHeightRatio);
    final double overlayTextFontSize =
        widgetSize.height * overlayFontSizeRatio;
    final double overlayTextStrokeWidth =
        widgetSize.height * textStrokeWidthRatio;
    return SizedBox(
      height: widget.size.height,
      width: widget.size.width,
      child: Stack(
        alignment: AlignmentDirectional.center,
        children: [
          _camera(),
          Align(
            alignment: Alignment.topRight,
            child: overlayText(cameraOverlayText, overlayTextContainerSize,
                overlayTextFontSize, overlayTextStrokeWidth),
          ),
        ],
      ),
    );
  }

  Widget _camera() {
    return FutureBuilder<void>(
      future: _initializeCameraController,
      builder: (context, snapshot) {
        if (snapshot.connectionState == ConnectionState.done) {
          return CameraPreview(_controller!);
        } else {
          if (_availableCamera) {
            return const Center(child: CircularProgressIndicator());
          } else {
            return _NoCameraOverlay(
              size: widget.size,
            );
          }
        }
      },
    );
  }

  void initCamera() async {
    final List<CameraDescription> cameras = await availableCameras();
    if (cameras.isEmpty) {
      _availableCamera = false;
      return;
    }

    _controller = CameraController(
      cameras[0],
      ResolutionPreset.medium,
      enableAudio: false,
    );

    _initializeCameraController = _controller?.initialize();
    setState(() {});
  }

  Future<void> onNewCameraSelected(CameraDescription cameraDescription) async {
    final CameraController? oldController = _controller;
    if (oldController != null) {
      // `controller` needs to be set to null before getting disposed,
      // to avoid a race condition when we use the controller that is being
      // disposed. This happens when camera permission dialog shows up,
      // which triggers `didChangeAppLifecycleState`, which disposes and
      // re-creates the controller.
      _controller = null;
      await oldController.dispose();
    }

    final CameraController cameraController = CameraController(
      cameraDescription,
      ResolutionPreset.medium,
      enableAudio: false,
    );

    _controller = cameraController;

    _initializeCameraController = cameraController.initialize();

    if (mounted) {
      setState(() {});
    }
  }
}

class _NoCameraOverlay extends StatelessWidget {
  const _NoCameraOverlay(
      {this.size = const Size(double.infinity, double.infinity)});

  final Size size;

  @override
  Widget build(BuildContext context) {
    return Container(
      color: noCameraOverlayBackgroundColor,
      child: Icon(
        Icons.no_photography,
        color: noCameraOverlayColor,
        size: size.width * noCameraOverlayIconRatio,
      ),
    );
  }
}
